
/* $Header: /cvsroot/faces/faces/faces/imap.h,v 1.1.1.1 2002/04/15 20:20:48 richburridge Exp $
 *
 * This module is a massivily modified version of imap_lib.c
 * found in the mutt mailreader.  The original copyrights read:
 *
 * Copyright (C) 2000 Vsevolod Volkov <vvv@mutt.org.ua>
 * 
 * Mutt is distributed under the FSF GNU General Public Licence.  The
 * complete Mutt sources and program can be found at http://www.mutt.org/.
 *
 * This version is
 *  Copyright (C) 2001-2002  Robert Adams (misterblue@misterblue.com)
 *  All rights reserved.
 *
 *  Permission is given to distribute these sources, as long as the
 *  copyright messages are not removed, and no monies are exchanged. 
 * 
 *  No responsibility is taken for any errors on inaccuracies inherent
 *  either to the comments or the code of this program, but if reported
 *  to me, then an attempt will be made to fix them.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef IMAP

#ifndef _IMAP_H
#define _IMAP_H 1

#include "face_socket.h"

#define IMAP_PORT 143

/* maximal length of the server response (RFC1939) */
#define IMAP_CMD_RESPONSE 512

enum {   /* Status. */
    IMAP_NONE = 0,
    IMAP_NONAUTHENTICATED,
    IMAP_CONNECTED,
    IMAP_DISCONNECTED,
    IMAP_BYE
};

typedef enum {
    IMAP_A_SUCCESS = 0,
    IMAP_A_SOCKET,
    IMAP_A_FAILURE,
    IMAP_A_UNAVAIL
} imap_auth_res_t;

/* types of command responses returned by imap_query */
/*   a response is TAG/CONTINUE/RESPOND or'ed with OK/BAD/NO */
#define IMAP_CMD_OK       (0x01)        /* * OK */
#define IMAP_CMD_BAD      (0x02)        /* * BAD */
#define IMAP_CMD_NO       (0x04)        /* * NO */
#define IMAP_CMD_RESPOND  (0x20)        /* * ... */
#define IMAP_CMD_CONTINUE (0x40)        /* + ... */
#define IMAP_CMD_TAG      (0x80)        /* A9999 ... */

#define TAGSIZE 5

typedef struct {
    CONNECTION *conn;
    char *account;
    char *passwd;
    unsigned int status;
    size_t size;
    int seqno;
    char err_msg[IMAP_CMD_RESPONSE];
    char cmdtag[TAGSIZE+1];
} IMAP_DATA;

/* imap.c */
IMAP_DATA* build_imapdata(char *, int, char *, char *);
int free_imapdata(IMAP_DATA *);

int imap_open_connection (IMAP_DATA *);
int imap_close_connection(IMAP_DATA *);
int imap_query(IMAP_DATA *, char *, int(*funct)(int, char*, void*), void*);
int imap_get_statinfo(IMAP_DATA*, long *, long *);

#endif /*IMAP_H*/

#endif /*IMAP*/
