
/* $Header: /cvsroot/faces/faces/faces/pop.c,v 1.1.1.1 2002/04/15 20:20:49 richburridge Exp $
 *
 * This module is a massivily modified version of pop_lib.c
 * found in the mutt mailreader.  The origional copyrights read:
 *
 * Copyright (C) 2000 Vsevolod Volkov <vvv@mutt.org.ua>
 * 
 * Mutt is distributed under the FSF GNU General Public Licence.  The
 * complete Mutt sources and program can be found at http://www.mutt.org/.
 *
 * This version is
 *  Copyright (C) 2001-2002  Robert Adams (misterblue@misterblue.com)
 *  All rights reserved.
 *
 *  Permission is given to distribute these sources, as long as the
 *  copyright messages are not removed, and no monies are exchanged. 
 * 
 *  No responsibility is taken for any errors on inaccuracies inherent
 *  either to the comments or the code of this program, but if reported
 *  to me, then an attempt will be made to fix them.
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

/* If not doing POP processing, ifdef the whole file out of existance */
#ifdef POP

#include <string.h>
#include <unistd.h>
#include <ctype.h>
#include <stdio.h>
#include <md5.h>

#include "faces.h"
#include "pop.h"
#include "face_socket.h"

#ifdef PDEBUG
#define  P_DEBUG(stmt)  stmt
#else
#define  P_DEBUG(stmt)
#endif /*PDEBUG*/

/* Function prototypes */
int pop_connect(POP_DATA *pop_data);
int pop_disconnect(POP_DATA *pop_data);
int pop_logout(POP_DATA *);
static int pop_authenticate(POP_DATA* pop_data);
int pop_auth_apop(POP_DATA *);
void pop_apop_timestamp(POP_DATA *, char *);
int pop_auth_user(POP_DATA *);
void pop_error(POP_DATA *pop_data, char *msg);

#define SKIPWS(c) while (*(c) && isspace ((unsigned char) *(c))) c++;


/* create and initialize a structure to hold the state of the POP
 * connection.  This lives over several requests so we can remember
 * how full the post box was last time.
 */

POP_DATA*
build_popdata(char* host, int port, char* acct, char* passwd)
{
    POP_DATA* popdata;

    popdata = (POP_DATA *)Malloc(sizeof(POP_DATA));
    CLEARMEM(popdata, sizeof(POP_DATA));
    popdata->conn = build_connection(host, port);

    /* Make sure both account and passwd have a string in them */
    if (acct) {
        popdata->account = strdup(acct);
    } else {
        popdata->account = strdup("");;
    }
    if (passwd) {
        popdata->passwd = strdup(passwd);
    } else {
        popdata->passwd = strdup("");
    }
    popdata->status = POP_NONE;
    popdata->size = 0;
    popdata->timestamp = NULL;

    return(popdata);
}


/* Release the POP_DATA structure and free what's in it */

int
free_popdata(POP_DATA* popdata)
{
    if (popdata) {
        if (popdata->conn) {
            face_socket_close(popdata->conn);
            free_connection(popdata->conn);
            popdata->conn = NULL;
        }
        if (popdata->account) {
            FREE(popdata->account);
            popdata->account = NULL;
        }
        if (popdata->passwd) {
            FREE(popdata->passwd);
            popdata->passwd = NULL;
        }
    }

    return(0);
}


/*
 * Open connection and authenticate
 *  0 - successful,
 * -1 - conection lost,
 * -2 - invalid command or execution error,
 * -3 - authentication canceled.
 */

int
pop_open_connection (POP_DATA *pop_data)
{
    int ret;

    /* connect to the mail server */
    ret = pop_connect(pop_data);
    if (ret < 0) {
        sleep (2);
        return(ret);
    }

    /* do the login thing */
    ret = pop_authenticate(pop_data);
    if (ret == -1) {
        goto err_conn;
    }
    if (ret != 0) {
        return(ret);
    }

    return(0);

err_conn:
    pop_data->status = POP_DISCONNECTED;
    FPRINTF(stderr, "Server closed connection!\n");
    sleep(2);

    return(-1);
}


/* Undo what was done by pop_open_connection */

int
pop_close_connection(POP_DATA* pop_data)
{
    int ret;

    if (pop_data->status == POP_CONNECTED) {
        ret = pop_logout(pop_data);
        ret = pop_disconnect(pop_data);
    }
    return 0;
}


/*
 * Open connection
 *  0 - successful,
 * -1 - conection lost,
 * -2 - invalid response.
 */

int
pop_connect (POP_DATA *pop_data)
{
    char buf[LONG_STRING];

    pop_data->status = POP_NONE;
    if (face_socket_open (pop_data->conn) < 0 ||
        face_socket_readln (pop_data->conn, buf, sizeof (buf)) < 0) {
        SPRINTF(buf,"Error connecting to server: %s", pop_data->conn->host);
        return(-1);
    }

    pop_data->status = POP_CONNECTED;

    if (strncmp(buf, "+OK", 3)) {
        *pop_data->err_msg = '\0';
        pop_error(pop_data, buf);
        P_DEBUG(FPRINTF(stderr, "%s\n", pop_data->err_msg));
        return(-2);
    }

    /* snatch the time seed for the APOP auth */
    pop_apop_timestamp(pop_data, buf);

    return(0);
}


/* Undo what was done by pop_connect */

int
pop_disconnect(POP_DATA* pop_data)
{
    int ret;

    if (pop_data->status != POP_NONE) {
        ret = face_socket_close(pop_data->conn);
        pop_data->status = POP_NONE;
    }

    return(0);
}


/* logout from POP server */

int
pop_logout(POP_DATA *popdata)
{
    int ret = 0;
    char buf[LONG_STRING];

    if (popdata->status == POP_CONNECTED) {
        P_DEBUG(IF_DEBUG(FPRINTF(stderr, 
                                 "Closing connection to POP server...\n")));

        ret = pop_query_c(popdata, "QUIT", buf, sizeof (buf));
    }

    popdata->status = POP_DISCONNECTED;

    return(0);
}


/* Copy error message to err_msg buffer in the pop_data
 * This will allow upper levels to print out reasonable error msgs.
 */

void
pop_error(POP_DATA *pop_data, char *msg)
{
    char *t, *c, *c2;

    t = strchr(pop_data->err_msg, '\0');
    c = msg;

    if (!strncmp(msg, "-ERR ", 5)) {
        c2 = msg + 5;
        SKIPWS (c2);

        if (*c2) {
            c = c2;
        }
    }

    strncpy(t, c, sizeof(pop_data->err_msg) - strlen(pop_data->err_msg));
}


/* Pass the user and password to the POP server and see if
 * we get logged in
 */

static int
pop_authenticate(POP_DATA* pop_data)
{
    int ret;

    /* try the APOP sign-in thing */
    ret = pop_auth_apop(pop_data);
    if (ret != POP_A_SUCCESS) {
        /* if APOP didn't work, try the insecure but basic USER/PASSWD */
        ret = pop_auth_user(pop_data);
        /* pass any error back to the caller */
    }

    return(ret);
}


/* authenticate using USER/PASS */

int
pop_auth_user(POP_DATA *pop_data)
{
    char buf[LONG_STRING];
    int ret;

    sprintf(buf, "USER %s\r\n", pop_data->account);
    ret = pop_query(pop_data, buf, sizeof(buf));
    if (ret == POP_A_SUCCESS) {
        /* it took the USER ... see if the PW is good */
        sprintf(buf, "PASS %s\r\n", pop_data->passwd);
        ret = pop_query(pop_data, buf, sizeof(buf));
        /* value of ret passed through -- if failed, this routine returns it */
    }

    return(ret);
}


/* authenticate using APOP */

int
pop_auth_apop(POP_DATA *pop_data)
{
    MD5_CTX mdContext;
    unsigned char digest[16];
    char hash[33];
    char buf[LONG_STRING];
    int i;

    P_DEBUG(FPRINTF(stderr, "pop_auth_apop: authenticating with APOP\n"));

    if (!pop_data->timestamp) {
        P_DEBUG(FPRINTF(stderr, 
                        "pop_auth_apop: no timestamp, could not auth\n"));
        return(POP_A_UNAVAIL);
    }

    /* Compute the authentication hash to send to the server */
    MD5Init(&mdContext);
    MD5Update(&mdContext, (unsigned char *)pop_data->timestamp,
              strlen(pop_data->timestamp));
    MD5Update(&mdContext, (unsigned char *)pop_data->passwd,
              strlen(pop_data->passwd));
    MD5Final(digest, &mdContext);

    /* turn digest into a hex string */
    for (i = 0; i < sizeof (digest); i++) {
        sprintf(hash + 2 * i, "%02x", digest[i]);
    }

    /* Send APOP command to server */
    sprintf(buf, "APOP %s %s\r\n", pop_data->account, hash);
    switch (pop_query(pop_data, buf, sizeof (buf))) {
        case 0:
            return(POP_A_SUCCESS);

        case -1:
            return(POP_A_SOCKET);
    }

    P_DEBUG(FPRINTF(stderr, "pop_auth_apop: APOP authentication failed\n"));

    return(POP_A_FAILURE);
}


/* Get the server timestamp for APOP authentication */

void
pop_apop_timestamp (POP_DATA *pop_data, char *buf)
{
    char *p1, *p2;

    if (pop_data->timestamp) {
        FREE(&pop_data->timestamp);
        pop_data->timestamp = NULL;
    }

    if ((p1 = strchr (buf, '<')) && (p2 = strchr (p1, '>'))) {
        p2[1] = '\0';
        pop_data->timestamp = strdup(p1);
        P_DEBUG(FPRINTF(stderr, 
                 "pop_apop_timestamp: timestamp = %s\n", pop_data->timestamp));
    }
}


/* Make a server query of the given command and return the
 * response in the passed buf.
 *  0 - successful,
 * -1 - conection lost,
 * -2 - invalid command or execution error.
 */

int
pop_query_c(POP_DATA *pop_data, char* cmd, char*buf, size_t buflen)
{
    strcpy(buf, cmd);
    strcat(buf, "\r\n");
    return(pop_query(pop_data, buf, buflen));
}


/*
 * Send data from buffer and receive answer to the same buffer
 *  0 - successful,
 * -1 - conection lost,
 * -2 - invalid command or execution error.
 */

int 
pop_query(POP_DATA *pop_data, char *buf, size_t buflen)
{
    if (pop_data->status != POP_CONNECTED) {
        return(-1);
    }

    face_socket_write(pop_data->conn, buf, strlen(buf));

    if (face_socket_readln(pop_data->conn, buf, buflen) < 0) {
        pop_data->status = POP_DISCONNECTED;
        return(-1);
    }
    if (!strncmp (buf, "+OK", 3)) {
        return(0);
    }

    pop_error (pop_data, buf);

    return(-2);
}


/* do a STAT query on the connection and get the size and count info */

int
pop_get_statinfo(POP_DATA* pop_data, long *msgs, long *boxsize)
{
    int ret;
    char buf[LONG_STRING];
    
    if (pop_data->status != POP_CONNECTED) {
        return(-1);
    }
    
    ret = pop_query_c(pop_data, "STAT", buf, sizeof(buf));
    if (ret != 0) {
        return(-1);
    }

    sscanf(buf, "+OK %ld %ld", msgs, boxsize);

    return(0);    
}


/*
 * This function calls  funct(*line, *data)  for each received line,
 * funct(NULL, *data)  if  rewind(*data)  needs, exits when fail or done.
 * Returned codes:
 *  0 - successful,
 * -1 - conection lost,
 * -2 - invalid command or execution error,
 * -3 - error in funct(*line, *data)
 */

int
pop_fetch_data(POP_DATA *pop_data, char *query, char *msg,
               int (*funct) (char *, void *), void *data)
{
    char buf[LONG_STRING];
    char *inbuf;
    char *p;
    int ret, chunk, iline = 0;
    size_t lenbuf = 0;

    ret = pop_query_c(pop_data, query, buf, sizeof (buf));
    if (ret < 0) {
        return(ret);
    }

    /* allocate the buffer that we will build */
    inbuf = (char*) Malloc(sizeof (buf));

    while (1) {
        /* read one chunk of the input stream */
        chunk = face_socket_readln(pop_data->conn, buf, sizeof(buf));
        if (chunk < 0) {
            pop_data->status = POP_DISCONNECTED;
            ret = -1;
            break;
        }

        /* terminator is a line with a dot.. if two dot's, remove "escape" 
         * dot (rfc1939).
         */
        p = buf;
        if (!lenbuf && buf[0] == '.') {
            if (buf[1] != '.') {
                break;
            }
            p++;
        }

        /* add the read buf to the inbuf we are building */
        STRNCPY(inbuf + lenbuf, p, sizeof(buf));

        /* is the chunk we read just part of  the line? */
        if (chunk >= sizeof (buf)) {
            /* if just part of the line, setup to append */
            lenbuf += strlen(p);
            if ((inbuf = (char*) realloc(inbuf, 
                                         lenbuf + sizeof(buf))) == NULL) {
                FPRINTF(stderr, "pop_fetch_data: Out of memory.\n");
            }
        } else {
            /* pass the read line to the caller's processing routine */
            iline++;
            P_DEBUG(FPRINTF(stderr, 
                "pop_fetch_data: calling usr funct: %d:'%s'\n", iline, inbuf));
            if (funct(inbuf, data) < 0) {
                ret = -3;
                break;
            }
            lenbuf = 0;
        }
    }

    FREE(inbuf);

    return(ret);
}
#endif /*POP*/
